path property
The static path of this border.
The path must be normalized to fit exactly into a single unit square, starting at (0, 0) with a bottom-right coordinate of (1, 1) respectively.
If squash is non-zero, the path will be scaled to match the aspect ratio of the bounding box of the widget that this border is attached to, which can give a squashed appearance.
Implementation
@protected
Path get path;